@@ -49,7 +49,6 @@ import org.jruby.RubySymbol;
 import org.jruby.anno.JRubyClass;
 import org.jruby.anno.JRubyMethod;
 import org.jruby.ast.util.ArgsUtil;
-import org.jruby.common.IRubyWarnings.ID;
 import org.jruby.exceptions.RaiseException;
 import org.jruby.runtime.ThreadContext;
 import org.jruby.runtime.builtin.IRubyObject;
@@ -70,7 +69,7 @@ import static org.jruby.runtime.Visibility.PRIVATE;
 public class RubyStringScanner extends RubyObject {
     private static final long serialVersionUID = -3722138049229128675L;
 
-    private static final String STRSCAN_VERSION = "3.1.6";
+    private static final String STRSCAN_VERSION = "3.1.7";
 
     private RubyString str;
     private int curr = 0;
@@ -83,24 +82,23 @@ public class RubyStringScanner extends RubyObject {
 
     public static RubyClass createScannerClass(final Ruby runtime) {
         RubyClass Object = runtime.getObject();
-        ThreadContext context = runtime.getCurrentContext();
 
         RubyClass scannerClass = runtime.defineClass("StringScanner", Object, RubyStringScanner::new);
 
         RubyClass standardError = runtime.getStandardError();
-        RubyClass error = scannerClass.defineClassUnder(context, "Error", standardError, standardError.getAllocator());
-        if (!Object.isConstantDefined(context, "ScanError")) {
-            Object.defineConstant(context, "ScanError", error);
-            Object.deprecateConstant(context, "ScanError");
+        RubyClass error = scannerClass.defineClassUnder("Error", standardError, standardError.getAllocator());
+        if (!Object.isConstantDefined("ScanError", true)) {
+            Object.defineConstant("ScanError", error);
+            Object.deprecateConstant(runtime, "ScanError");
         }
 
         RubyString version = runtime.newString(STRSCAN_VERSION);
         version.setFrozen(true);
-        scannerClass.setConstant(context, "Version", version);
+        scannerClass.setConstant("Version", version);
         RubyString id = runtime.newString("$Id$");
         id.setFrozen(true);
-        scannerClass.setConstant(context, "Id", id);
-        scannerClass.deprecateConstant(context, "Id");
+        scannerClass.setConstant("Id", id);
+        scannerClass.deprecateConstant(runtime, "Id");
 
         scannerClass.defineAnnotatedMethods(RubyStringScanner.class);
 
@@ -186,7 +184,7 @@ public class RubyStringScanner extends RubyObject {
     @JRubyMethod(name = "terminate")
     public IRubyObject terminate(ThreadContext context) {
         check(context);
-        curr = str.getByteList().getRealSize();
+        curr = str.size();
         clearMatchStatus();
         return this;
     }
@@ -205,9 +203,9 @@ public class RubyStringScanner extends RubyObject {
     }
 
     @JRubyMethod(name = {"concat", "<<"})
-    public IRubyObject concat(ThreadContext context, IRubyObject obj) {
+    public IRubyObject concat(ThreadContext context, IRubyObject str) {
         check(context);
-        str.append(obj.convertToString());
+        this.str.append(RubyString.stringValue(str));
         return this;
     }
 
@@ -224,7 +222,7 @@ public class RubyStringScanner extends RubyObject {
         Ruby runtime = context.runtime;
 
         int i = RubyNumeric.num2int(pos);
-        int size = str.getByteList().getRealSize();
+        int size = str.size();
         if (i < 0) i += size;
         if (i < 0 || i > size) throw runtime.newRangeError("index out of range.");
         this.curr = i;
@@ -243,8 +241,7 @@ public class RubyStringScanner extends RubyObject {
     }
 
     private IRubyObject extractRange(Ruby runtime, int beg, int end) {
-        ByteList byteList = str.getByteList();
-        int size = byteList.getRealSize();
+        int size = str.size();
 
         if (beg > size) return runtime.getNil();
         if (end > size) end = size;
@@ -255,7 +252,7 @@ public class RubyStringScanner extends RubyObject {
     private IRubyObject extractBegLen(Ruby runtime, int beg, int len) {
         assert len >= 0;
 
-        int size = str.getByteList().getRealSize();
+        int size = str.size();
 
         if (beg > size) return runtime.getNil();
         len = Math.min(len, size - beg);
@@ -264,7 +261,7 @@ public class RubyStringScanner extends RubyObject {
     }
 
     // MRI: strscan_do_scan
-    private IRubyObject scan(ThreadContext context, IRubyObject regex, boolean succptr, boolean getstr, boolean headonly) {
+    private IRubyObject scan(ThreadContext context, IRubyObject pattern, boolean succptr, boolean getstr, boolean headonly) {
         final Ruby runtime = context.runtime;
         check(context);
         clearMatchStatus();
@@ -277,12 +274,12 @@ public class RubyStringScanner extends RubyObject {
         ByteList strBL = str.getByteList();
         int currPtr = strBL.getBegin() + curr;
 
-        if (regex instanceof RubyRegexp) {
-            pattern = ((RubyRegexp) regex).preparePattern(str);
+        if (pattern instanceof RubyRegexp) {
+            this.pattern = ((RubyRegexp) pattern).preparePattern(str);
 
             int range = currPtr + restLen;
 
-            Matcher matcher = pattern.matcher(strBL.getUnsafeBytes(), matchTarget(), range);
+            Matcher matcher = this.pattern.matcher(strBL.getUnsafeBytes(), matchTarget(), range);
             final int ret;
             if (headonly) {
                 ret = RubyRegexp.matcherMatch(context, matcher, currPtr, range, Option.NONE);
@@ -297,26 +294,28 @@ public class RubyStringScanner extends RubyObject {
                 regs = matchRegion;
             }
 
-            if  (ret == -2) {
-                throw runtime.newRaiseException((RubyClass) getMetaClass().getConstant("ScanError"), "regexp buffer overflow");
+            if (ret < 0) { // MISMATCH
+                return context.nil;
             }
-            if (ret < 0) return context.nil;
         } else {
-            RubyString pattern = regex.convertToString();
-            Encoding patternEnc = str.checkEncoding(pattern);
-            ByteList patternBL = pattern.getByteList();
-            int patternSize = patternBL.realSize();
+            RubyString patternStr = RubyString.stringValue(pattern);
+            ByteList patternBL = patternStr.getByteList();
+            final int patternSize = patternBL.realSize();
+
+            if (restLen < patternSize) {
+                str.checkEncoding(patternStr);
+                return context.nil;
+            }
 
             if (headonly) {
-                if (restLen < pattern.size()) {
-                    return context.nil;
-                }
+                str.checkEncoding(patternStr);
+
                 if (ByteList.memcmp(strBL.unsafeBytes(), currPtr, patternBL.unsafeBytes(), patternBL.begin(), patternSize) != 0) {
                     return context.nil;
                 }
                 setRegisters(0, patternSize);
             } else {
-                int pos = StringSupport.index(strBL, patternBL, currPtr, patternEnc);
+                int pos = StringSupport.index(strBL, patternBL, currPtr, str.checkEncoding(patternStr));
                 if (pos == -1) {
                     return context.nil;
                 }
@@ -356,10 +355,6 @@ public class RubyStringScanner extends RubyObject {
         }
     }
 
-    private int currPtr() {
-        return str.getByteList().getBegin() + curr;
-    }
-
     private int matchTarget() {
         if (fixedAnchor) {
             return str.getByteList().getBegin();
@@ -484,7 +479,7 @@ public class RubyStringScanner extends RubyObject {
     public IRubyObject get_byte(ThreadContext context) {
         check(context);
         clearMatchStatus();
-        if (curr >= str.getByteList().getRealSize()) return context.nil;
+        if (curr >= str.size()) return context.nil;
 
         prev = curr;
         curr++;
@@ -712,7 +707,7 @@ public class RubyStringScanner extends RubyObject {
 
         if (idx instanceof RubySymbol || idx instanceof RubyString) {
             if (pattern == null) {
-                throw runtime.newRaiseException((RubyClass) getMetaClass().getConstant("IndexError"), "undefined group name reference: " + idx);
+                throw runtime.newRaiseException(runtime.getIndexError(), "undefined group name reference: " + idx);
             }
         }
 
@@ -761,8 +756,7 @@ public class RubyStringScanner extends RubyObject {
         check(context);
         Ruby runtime = context.runtime;
 
-        ByteList value = str.getByteList();
-        int realSize = value.getRealSize();
+        int realSize = str.size();
 
         if (curr >= realSize) {
             return RubyString.newEmptyString(runtime, str.getEncoding());
@@ -951,12 +945,25 @@ public class RubyStringScanner extends RubyObject {
 
     // MRI: str_new
     private RubyString newString(Ruby runtime, int start, int length) {
-        ByteList byteList = str.getByteList();
-        int begin = byteList.begin();
+        final ByteList strBytes = this.str.getByteList();
+        ByteList newBytes = new ByteList(strBytes.unsafeBytes(), strBytes.begin() + start, length, true);
 
-        ByteList newByteList = new ByteList(byteList.unsafeBytes(), begin + start, length, byteList.getEncoding(), true);
+        final RubyString newStr = RubyString.newString(runtime, newBytes, strBytes.getEncoding());
+        copyCodeRangeForSubstr(newStr, this.str);
+        return newStr;
+    }
 
-        return RubyString.newString(runtime, newByteList);
+    /**
+     * Same as JRuby's (private) <code>RubyString#copyCodeRangeForSubstr</code>.
+     * Isn't really necessary, but will avoid extra code-range scans for the substrings returned.
+     */
+    private void copyCodeRangeForSubstr(RubyString str, RubyString from) {
+        if (str.size() == 0) {
+            str.setCodeRange(from.getEncoding().isAsciiCompatible() ? StringSupport.CR_7BIT : StringSupport.CR_VALID);
+        } else {
+            if (from.getCodeRange() == StringSupport.CR_7BIT) str.setCodeRange(StringSupport.CR_7BIT);
+            // otherwise, leave it as CR_UNKNOWN
+        }
     }
 
     /**
